/**
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 *
 * Пример зашифрования/расшифрования PKCS7 (detached/attached).
 * Составлен из примеров CMS, CMSDcrypt, CMSSignAndEncrypt. 
 * Требуется наличие набора примеров samples.jar.
 */
package CMS_samples;

import com.objsys.asn1j.runtime.Asn1BerDecodeBuffer;
import com.objsys.asn1j.runtime.Asn1BerEncodeBuffer;
import com.objsys.asn1j.runtime.Asn1ObjectIdentifier;
import com.objsys.asn1j.runtime.Asn1OctetString;

import ru.CryptoPro.Crypto.CryptoProvider;

import ru.CryptoPro.JCP.ASN.CryptographicMessageSyntax.*;
import ru.CryptoPro.JCP.ASN.Gost28147_89_EncryptionSyntax.*;
import ru.CryptoPro.JCP.ASN.GostR3410_EncryptionSyntax.GostR3410_KeyTransport;
import ru.CryptoPro.JCP.ASN.GostR3410_EncryptionSyntax.GostR3410_TransportParameters;
import ru.CryptoPro.JCP.ASN.PKIX1Explicit88.*;

import ru.CryptoPro.JCP.JCP;
import ru.CryptoPro.JCP.Key.GostPublicKey;
import ru.CryptoPro.JCP.params.*;
import ru.CryptoPro.JCP.spec.GostCipherSpec;
import ru.CryptoPro.JCP.tools.Array;

import javax.crypto.Cipher;
import javax.crypto.KeyAgreement;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.security.auth.x500.X500Principal;

import java.security.*;
import java.security.cert.X509Certificate;

/**
 *	*.txt - файл с исходной текстовой информацией.
 *	*.cms - файл с подписью PKCS7 (DETACHED/ATTACHED).
 *	*.env - файл с зашифрованной подписью PKCS7 (ENVELOPED).
 *	client_exch - сертификат отправителя.
 *	afevma_gost_exch_server - сертификат получателя.
 *	Формирование PKCS7 DETACHED:
 *		csptest -cmssfsign -sign -in "in.txt" -my client_exch -add -detached -signature "out.cms"
 *	Проверка PKCS7 CMS DETACHED:
 *		csptest -cmssfsign -verify -in "in.txt" -my client_exch -add -detached -signature "out.cms"
 *	Формирование PKCS7 CMS ATTACHED:
 *		csptest -cmssfsign -sign -in "in.txt" -my client_exch -add -signature "out.cms"
 *	Проверка PKCS& CMS ATTACHED:
 *		csptest -cmssfsign -verify -in "out.cms" -my client_exch -add
 *	Зашифрование PKCS7 CMS DETACHED/ATTACHED:
 *		csptest -sfenc -encrypt -out "in.cms" -in "pkcs7.env" -cert client_exch -my afevma_gost_exch_server
 *	Расшифрование PKCS7 CMS DETACHED/ATTACHED:
 *		csptest -sfenc -decrypt -out "out.cms" -in "pkcs7.env" -cert afevma_gost_exch_server -my client_exch
 *	Подпись и зашифование PKCS7 CMS ATTACHED:
 *		csptest -sfse -encrypt -in "in.txt" -out "pkcs7.env" -senderDN client_exch -recipDN afevma_gost_exch_server
 *	Проверка и расшифрование PKCS7 CMS ATTACHED:
 *		csptest -sfse -decrypt -in "pkcs7.env" -out "out.txt" -senderDN client_exch -recipDN afevma_gost_exch_server
 */
public class PKCS7EnvEphTransport {

	/**
	 * Алиас получателя.
	 */
	private final static String RECIPIENT_ALIAS =
		// "GostUsr";
		"gost_exch";
		// "cdsUr01";
	
	/**
	 * Пароль к контейнеру получателя (при расшифровании).
	 */
	private final static char[] RECIPIENT_PASSWORD =
		// null;
		"Pass1234".toCharArray();
		// "c1".toCharArray();
	
	/**
	 * Папка с файлами.
	 */
	private final static String TEST_DIR = "C:\\Temp\\pkcs7\\new\\";

	/**
	 * Режим шифрования.
	 */
	private final static String CIPHER = "GOST28147";

	/**
	 * Режим шифрования данных.
	 */
	private final static String CIPHER_MODE = CIPHER + "/CFB/NoPadding";

	/**
	 * Провайдер хранилища, подписи, хеширования.
	 */
	private final static String PROVIDER_NAME = JCP.PROVIDER_NAME;

	/**
	 * Провайдер шифрования.
	 */
	private final static String CRYPT_PROVIDER_NAME = CryptoProvider.PROVIDER_NAME;

	/**
	 * Тип контейнера.
	 */
	private final static String STORE_TYPE = JCP.HD_STORE_NAME;

	/**
	 * Хранилище контейнеров.
	 */
	private final static KeyStore keyStore;

	static {
		keyStore = loadKeyStore();
	}
	
	/**
	 * Загрузка контейнеров.
	 * @return указатель на загруженные контейнеры.
	 */
	public static KeyStore loadKeyStore() {

		try {

			KeyStore keyStore = KeyStore.getInstance(STORE_TYPE, PROVIDER_NAME);
			keyStore.load(null, null);

			return keyStore;

		} catch (Exception e) {
			throw new RuntimeException(e);
		}

	}

	/**
	 * Зашифрование PKCS7 (Enveloped).
	 *
	 * @param recipientAlias - алиас получателя (сертификат).
	 * @param pkcs7 - сообщение PKCS7.
	 * @return зашифрованное PKCS7 (Enveloped).
	 * @throws Exception
	 */
	public static byte[] EncryptPKCS7(String recipientAlias, byte[] pkcs7) throws Exception {

		// Сертификат получателя.
		final X509Certificate recipientCert = (X509Certificate) keyStore.getCertificate(recipientAlias);
		return EncryptPKCS7(recipientCert, pkcs7);

	}
	
	/**
	 * Зашифрование PKCS7 (Enveloped).
	 *
	 * @param recipientCert - сертификат получателя.
	 * @param pkcs7 - сообщение PKCS7.
	 * @return зашифрованное PKCS7 (Enveloped).
	 * @throws Exception
	 */
	public static byte[] EncryptPKCS7(X509Certificate recipientCert, byte[] pkcs7) throws Exception {

		final PublicKey recipientPublic = recipientCert.getPublicKey();

		// Генерирование симметричного ключа с параметрами
		// шифрования из контрольной панели.

		final KeyGenerator kg = KeyGenerator.getInstance(CMStools.SEC_KEY_ALG_NAME, CRYPT_PROVIDER_NAME);
		final ParamsInterface paramss = AlgIdSpec.getDefaultCryptParams();

		kg.init(paramss);
		final SecretKey simm = kg.generateKey();
	    
		// Зашифрование текста на симметричном ключе.

		Cipher cipher = Cipher.getInstance(CIPHER_MODE, CRYPT_PROVIDER_NAME);
		cipher.init(Cipher.ENCRYPT_MODE, simm, (SecureRandom) null);

		final byte[] iv = cipher.getIV();
		final byte[] text = cipher.doFinal(pkcs7, 0, pkcs7.length);

		// Зашифрование симметричного ключа.

		final byte[] keyTransport = wrap(simm, recipientPublic);

		// Формирование CMS-сообщения.

		final ContentInfo all = new ContentInfo();
		all.contentType = new Asn1ObjectIdentifier(new OID(CMStools.STR_CMS_OID_ENVELOPED).value);

		final EnvelopedData cms = new EnvelopedData();

		all.content = cms;
		cms.version = new CMSVersion(0);

		cms.recipientInfos = new RecipientInfos(1);
		cms.recipientInfos.elements = new RecipientInfo[1];
		cms.recipientInfos.elements[0] = new RecipientInfo();

		final KeyTransRecipientInfo keytrans = new KeyTransRecipientInfo();
		keytrans.version = new CMSVersion(0);

		final Asn1BerEncodeBuffer ebuf = new Asn1BerEncodeBuffer();

		final AlgIdInterface algid = new AlgIdSpec(JCP.GOST_EL_KEY_OID);
		final AlgorithmIdentifier id = (AlgorithmIdentifier) algid.getDecoded();
		id.encode(ebuf);

		Asn1BerDecodeBuffer dbuf = new Asn1BerDecodeBuffer(ebuf.getMsgCopy());
		keytrans.keyEncryptionAlgorithm = new KeyEncryptionAlgorithmIdentifier();
		keytrans.keyEncryptionAlgorithm.decode(dbuf);

		ebuf.reset();
		dbuf.reset();
	    
		keytrans.rid = new RecipientIdentifier();
		final IssuerAndSerialNumber issuer = new IssuerAndSerialNumber();

		final X500Principal issuerName = recipientCert.getIssuerX500Principal();
		dbuf = new Asn1BerDecodeBuffer(issuerName.getEncoded());
	    
		issuer.issuer = new Name();
		final RDNSequence rnd = new RDNSequence();
		rnd.decode(dbuf);
	    
		issuer.issuer.set_rdnSequence(rnd);
		issuer.serialNumber = new CertificateSerialNumber(recipientCert.getSerialNumber());

		keytrans.rid.set_issuerAndSerialNumber(issuer);
		dbuf.reset();

		keytrans.encryptedKey = new EncryptedKey(keyTransport);
		ebuf.reset();
	    
		cms.recipientInfos.elements[0].set_ktri(keytrans);
		cms.encryptedContentInfo = new EncryptedContentInfo();

		final OID contentType = new OID(CMStools.STR_CMS_OID_DATA);
		cms.encryptedContentInfo.contentType = new ContentType(contentType.value);

		final Gost28147_89_Parameters params = new Gost28147_89_Parameters();
		params.iv = new Gost28147_89_IV(iv);

		params.encryptionParamSet = new Gost28147_89_ParamSet(paramss.getOID().value);
		cms.encryptedContentInfo.contentEncryptionAlgorithm = new ContentEncryptionAlgorithmIdentifier(
			_Gost28147_89_EncryptionSyntaxValues.id_Gost28147_89, params);

		cms.encryptedContentInfo.encryptedContent = new EncryptedContent(text);
		all.encode(ebuf);

		return ebuf.getMsgCopy();
	}
		
	/**
	 * Расшифрование PKCS7 (Enveloped).
	 *
	 * @param recipientAlias - алиас получателя.
	 * @param recipientPassword - пароль получателя.
	 * @param pkcs7Env - зашифрованное PKCS7 (Enveloped).
	 * @param inData - исходные данные. Нужны при проверке detached подписи.
	 * @param detached - флаг detached подписи.
	 * @return результат проверки.
	 * @throws Exception
	 */
	public static boolean DecryptPKCS7(String recipientAlias, char[] recipientPassword,
		byte[] pkcs7Env, byte[] inData, boolean detached) throws Exception {

		// Разбор CMS-сообщения.

		Asn1BerDecodeBuffer dbuf = new Asn1BerDecodeBuffer(pkcs7Env);
		final ContentInfo all = new ContentInfo();

		all.decode(dbuf);
		dbuf.reset();
		
		final EnvelopedData cms = (EnvelopedData) all.content;

		KeyTransRecipientInfo keytrans = new KeyTransRecipientInfo();
		if (cms.recipientInfos.elements[0].getChoiceID() == RecipientInfo._KTRI) {
			keytrans = (KeyTransRecipientInfo) (cms.recipientInfos.elements[0].getElement());
		}
		else {
			throw new Exception("Unknown recipient info");
		}

		final byte[] wrapKey = keytrans.encryptedKey.value;
		final Gost28147_89_Parameters params = (Gost28147_89_Parameters)
            cms.encryptedContentInfo.contentEncryptionAlgorithm.parameters;

		final byte[] iv = params.iv.value;
		final OID cipherOID = new OID(params.encryptionParamSet.value);
		final byte[] text = cms.encryptedContentInfo.encryptedContent.value;

		// Получатель - закрытый ключ.
		final PrivateKey recipientKey = (PrivateKey) keyStore.getKey(recipientAlias, recipientPassword);

		// Выработка ключа согласования получателем и
		// расшифрование симметричного ключа.

		final SecretKey simmKey = unwrap(wrapKey, recipientKey);

		// Расшифрование текста на симметричном ключе.

		final GostCipherSpec spec = new GostCipherSpec(iv, cipherOID);
		Cipher cipher = Cipher.getInstance(CIPHER_MODE, CRYPT_PROVIDER_NAME);

		cipher.init(Cipher.DECRYPT_MODE, simmKey, spec, null);
		final byte[] result = cipher.doFinal(text, 0, text.length);

		Array.writeFile(TEST_DIR + "cms_cms.bin", result);
		// checkPKCS7(result, detached, inData, TEST_DIR + "cms_cms_data.txt");

		return true;

	}

	/**
	 * Зашифрование сессионного ключа.
	 *
	 * @param secretKey Сессионный ключ.
	 * @param recipientKey Открытый ключ получателя.
	 * @return транспортная структура GostR3410_KeyTransport.
	 * @throws Exception
	 */
	private static byte[] wrap(SecretKey secretKey, Key recipientKey) throws Exception {

		// Генерация эфемерной пары.

		KeyPairGenerator kgp = KeyPairGenerator.getInstance(
			JCP.GOST_EL_DH_EPH_NAME,
			CRYPT_PROVIDER_NAME
		);

		// Устанавливаем нужные параметры, как у получателя.

		GostPublicKey repPublicKey = (GostPublicKey) recipientKey;
		ParamsInterface repPublicKeySpec = repPublicKey.getSpec().getParams();
		kgp.initialize(repPublicKeySpec);

		// Генерируем эфемерную пару.

		KeyPair ephPair = kgp.generateKeyPair();

		PrivateKey privateKey = ephPair.getPrivate();
		PublicKey publicKey = ephPair.getPublic();

		byte[] syncro = new byte[8];
		SecureRandom random = SecureRandom.getInstance(JCP.CP_RANDOM, PROVIDER_NAME);
		random.nextBytes(syncro);

		IvParameterSpec iv = new IvParameterSpec(syncro);
		OID oid = CryptParamsSpec.OID_Crypt_VerbaO;

		// Выработка ключа согласования.

		KeyAgreement ka = KeyAgreement.getInstance(privateKey.getAlgorithm(), CRYPT_PROVIDER_NAME);
		ka.init(privateKey, iv);

		ka.doPhase(recipientKey, true);
		Key dh = ka.generateSecret(CIPHER);

		//Зашифрование симметричного ключа на ключе согласования
		// отправителя.

		final Cipher cipher = Cipher.getInstance(CIPHER, CRYPT_PROVIDER_NAME);
		cipher.init(Cipher.WRAP_MODE, dh, (SecureRandom) null);

		final byte[] wrappedKey = cipher.wrap(secretKey);

		// Упаковка параметров и ключа.

		Gost28147_89_EncryptedKey encryptedKey = new Gost28147_89_EncryptedKey();
		Asn1BerDecodeBuffer decoder = new Asn1BerDecodeBuffer(wrappedKey);
		encryptedKey.decode(decoder);

		byte[] imita = encryptedKey.macKey.value;
		byte[] wrapperKeyBytes = encryptedKey.encryptedKey.value;

		// Кодирование открытого ключа в SubjectPublicKeyInfo.

		byte[] publicKeyBytes = publicKey.getEncoded();
		SubjectPublicKeyInfo publicKeyInfo = new SubjectPublicKeyInfo();

		decoder = new Asn1BerDecodeBuffer(publicKeyBytes);
		publicKeyInfo.decode(decoder);

		// Кодирование GostR3410_KeyTransport.

		GostR3410_KeyTransport keyTransport = new GostR3410_KeyTransport();
		Asn1BerEncodeBuffer encoder = new Asn1BerEncodeBuffer();

		keyTransport.sessionEncryptedKey = new Gost28147_89_EncryptedKey(wrapperKeyBytes, imita);
		keyTransport.transportParameters = new GostR3410_TransportParameters(
			new Gost28147_89_ParamSet(oid.value),
			publicKeyInfo,
			new Asn1OctetString(iv.getIV()));

		keyTransport.encode(encoder);
		return encoder.getMsgCopy();

	}

	/**
	 * Зашифрование сессионного ключа.
	 *
	 * @param wrappedKey Зашифрованный сессионный ключ (транспортная
	 * структура GostR3410_KeyTransport).
	 * @param recipientKey Закрытый ключ получателя.
	 * @return сессионный ключ.
	 * @throws Exception
	 */
	private static SecretKey unwrap(byte[] wrappedKey, Key recipientKey) throws Exception {

		// Декодирование GostR3410_KeyTransport.

		GostR3410_KeyTransport keyTransport = new GostR3410_KeyTransport();
		Asn1BerDecodeBuffer decoder = new Asn1BerDecodeBuffer(wrappedKey);
		keyTransport.decode(decoder);

		// EncryptedKey в правильном формате для шифратора.

		byte[] wrappedKeyBytes = keyTransport.sessionEncryptedKey.encryptedKey.value;
		byte[] imita = keyTransport.sessionEncryptedKey.macKey.value;

		Asn1BerEncodeBuffer encoder = new Asn1BerEncodeBuffer();
		Gost28147_89_EncryptedKey encryptedKey = new Gost28147_89_EncryptedKey(
			new Gost28147_89_Key(wrappedKeyBytes),
			null, new Gost28147_89_MAC(imita));

		encryptedKey.encode(encoder);
		byte[] wrapped = encoder.getMsgCopy();

		// Декодирование открытого ключа.

		encoder.reset();
		keyTransport.transportParameters.ephemeralPublicKey.encode(encoder);

		byte[] encodedPublic = encoder.getMsgCopy();
		PublicKey publicKey = new GostPublicKey(encodedPublic, false); // не из сертификата

		// Параметры шифрования.

		IvParameterSpec iv = new IvParameterSpec(keyTransport.transportParameters.ukm.value);
		OID oid = new OID(keyTransport.transportParameters.encryptionParamSet.value);

		CryptParamsSpec uz = CryptParamsSpec.getInstance(oid);
		GostCipherSpec params = new GostCipherSpec(iv, uz);

		// Выработка ключа согласования.

		KeyAgreement ka = KeyAgreement.getInstance(recipientKey.getAlgorithm(), CRYPT_PROVIDER_NAME);
		ka.init(recipientKey, iv);

		ka.doPhase(publicKey, true);
		Key dh = ka.generateSecret(CIPHER);

		// Расшифрование сессионного ключа.

		Cipher cipher = Cipher.getInstance(CIPHER, CRYPT_PROVIDER_NAME);
		cipher.init(Cipher.UNWRAP_MODE, dh, params);

		return (SecretKey) cipher.unwrap(wrapped, null, Cipher.SECRET_KEY);

	}

	/**
	 * Проверка PKCS7 подписи.
	 *
	 * @param cms - расшифрованная подпись.
	 * @param detached  - флаг detached подписи.
	 * @param data - исходные данные. Нужны при проверке detached подписи.
	 * @param contentFile - файл для сохранения исходных данных.
	 * @throws Exception
	 */
	private static void checkPKCS7(byte[] cms, boolean detached, byte[] data,
		String contentFile) throws Exception {

		// 1. Проверка подписи PKCS7.

		CMSVerify.CMSVerifyEx(cms, null, detached ? data : null,
				JCP.GOST_DIGEST_OID, JCP.GOST_DIGEST_NAME,
				JCP.GOST_EL_SIGN_NAME, PROVIDER_NAME);

		// 2. Извлечение текста, если он есть (для attached).

		final Asn1BerDecodeBuffer asnBuf = new Asn1BerDecodeBuffer(cms);
		final ContentInfo cInfo = new ContentInfo();
		cInfo.decode(asnBuf);
		final SignedData pkcs7 = (SignedData) cInfo.content;

		// 2. Сохранение текста в файл (для attached).

		if (pkcs7.encapContentInfo.eContent != null) {

			byte[] content = pkcs7.encapContentInfo.eContent.value;
			Array.writeFile(contentFile + ".data_content.txt", content);

			if (!Array.compare(content, data)) {
				throw new Exception("Invalid content");
			} // if

		} // if

	}

	/**
	 * Выполнение создания PKCS7 подписи, зашифрования, расшифрования и
	 * проверки подписи.
	 *
	 * @throws Exception
	 */
	private static void test_SignEncrypt_DecryptVerify() throws Exception {

		// Данные для attached подписи. Нужны, если detached = true.
		//final byte[] data = Array.readFile(TEST_DIR + "in.txt");

		// Пусть подпишет получатель, у нас есть только его ключ.
		//PrivateKey signerKey = (PrivateKey) keyStore.getKey(RECIPIENT_ALIAS, RECIPIENT_PASSWORD);
		//X509Certificate signerCert = (X509Certificate) keyStore.getCertificate(RECIPIENT_ALIAS);

		// 1. Создание PKCS7.
	   /*
		CMSSign.createCMSEx(
				data,
				new PrivateKey[]{signerKey},
				new Certificate[]{signerCert},
				TEST_DIR + "jcp_attached_cms.bin",
				false,
				JCP.GOST_DIGEST_OID,
				JCP.GOST_EL_SIGN_OID,
				JCP.GOST_EL_SIGN_NAME,
				PROVIDER_NAME);
				*/
		// "C:\Program Files\Crypto Pro\CSP\csptest" -cmssfsign -sign -in "in.txt" -my signencr -add -out "csp_attached_cms.bin" (создание CSP)
		// "C:\Program Files\Crypto Pro\CSP\csptest" -cmssfsign -verify -in "cms_cms.bin" -my signencr (проверка подписи JCP с помощью CSP)

		// Файл с PKCS7 для зашифрования.
		final byte[] pkcs7Data = "Hello".getBytes(); // Array.readFile(TEST_DIR + "jcp_attached_cms.bin");

		// 2. Зашифрование PKCS7.

		//X509Certificate recipientCert = (X509Certificate) CertificateFactory
		//	.getInstance("X.509").generateCertificate(new FileInputStream(
		//		"c:\\Support\\mironov\\primer JCP\\primer JCP.cer"));
		//
		byte[] pkcs7Env =  EncryptPKCS7(RECIPIENT_ALIAS, pkcs7Data);
		// "C:\Program Files\Crypto Pro\CSP\csptest" -sfenc -encrypt -in "csp_attached_cms.bin" -out "csp_enveloped_cms.bin" -cert signenc (создание CSP)

		Array.writeFile(TEST_DIR + "jcp_enveloped_cms.bin", pkcs7Env);

		// 3*. Расшифрование PKCS7 и проверка подписи.

		byte[] pkcs7EnvBack = Array.readFile(TEST_DIR + "jcp_enveloped_cms.bin.enc");
		System.out.println( "Verified: " + DecryptPKCS7(RECIPIENT_ALIAS,
			RECIPIENT_PASSWORD, pkcs7EnvBack, null, false) );
		// "C:\Program Files\Crypto Pro\CSP\csptest" -sfse -decrypt -in "jcp_enveloped_cms.bin" -out "csp_jcp_cms_data_content.txt" (проверка JCP с помощью CSP)
		// "C:\Program Files\Crypto Pro\CSP\csptest" -sfse -decrypt -in "csp_enveloped_cms.bin" -out "csp_cms_data_content.txt" (проверка CSP с помощью CSP)
		// content: *_content.txt == in.txt

	}

	/**
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {
		test_SignEncrypt_DecryptVerify();
	}
}
