/**
 * Copyright 2004-2013 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp.client.example;

import android.util.Log;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.SingleClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;

import ru.CryptoPro.ACSPClientApp.client.example.interfaces.ThreadExecuted;
import ru.CryptoPro.ACSPClientApp.util.Constants;
import ru.CryptoPro.ACSPClientApp.util.ContainerAdapter;
import ru.CryptoPro.ACSPClientApp.util.KeyStoreType;
import ru.CryptoPro.ACSPClientApp.util.Logger;

import ru.CryptoPro.JCSP.JCSP;
import ru.CryptoPro.ssl.android.Provider;

import java.net.URI;
import java.security.KeyStore;

/**
 * Класс HttpTLSExample реализует пример обмена
 * по TLS 1.0 с помощью apache http client.
 *
 * 30/05/2013
 *
 */
public class HttpTLSExample extends TLSExample {

    /**
     * Конструктор.
     *
     * @param adapter Настройки примера.
     */
    protected HttpTLSExample(ContainerAdapter adapter) {
        super(adapter);
    }

    @Override
    public void getResult() throws Exception {
        getThreadResult(new HttpTLSThread());
    }

    /**
     * Класс SimpleTLSThread реализует подключение
     * apache http клиента по TLS в отдельном потоке.
     *
     */
    private class HttpTLSThread implements ThreadExecuted {

        @Override
        public void execute() throws Exception {

            containerAdapter.printConnectionInfo();
            HttpClient httpClient = null;

            try {

                Logger.log("Init Apache Http Client Example.");

                final String httpAddress = containerAdapter
                    .getConnectionInfo().toUrl();

                URI url = new URI(httpAddress);

                // Хранилища для подключения.

                Logger.log("Load trusted stores.");

                /**
                 * Для чтения(!) доверенного хранилища доступна
                 * реализация CertStore из Java CSP. В ее случае
                 * можно не использовать пароль.
                 */

                KeyStore ts = KeyStore.getInstance(
                    containerAdapter.getTrustStoreType(),
                    containerAdapter.getTrustStoreProvider());

                ts.load(containerAdapter.getTrustStoreStream(),
                    containerAdapter.getTrustStorePassword());

                KeyStore ks = null;
                if (containerAdapter.isUseClientAuth()) {

                    // Тип контейнера по умолчанию.
                    String keyStoreType = KeyStoreType.currentType();
                    Logger.log("Load key stores. Default container " +
                        "type: " + keyStoreType);

                    ks = KeyStore.getInstance(keyStoreType, JCSP.PROVIDER_NAME);
                    ks.load(null, null);

                } // if

                String keyStorePasswordValue =
                    (containerAdapter.getClientPassword() == null)
                    ? null : String.valueOf(containerAdapter.getClientPassword());

                Logger.log("Create socket factory.");

                SSLSocketFactory socketFactory = new SSLSocketFactory(
                    Provider.ALGORITHM, ks, keyStorePasswordValue, ts, null, null);

                socketFactory.setHostnameVerifier(
                    SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);

                Logger.log("Register https scheme.");

                // Регистрируем HTTPS схему.

                Scheme httpsScheme = new Scheme("https", socketFactory,
                    containerAdapter.getConnectionInfo().getHostPort());

                SchemeRegistry schemeRegistry = new SchemeRegistry();
                schemeRegistry.register(httpsScheme);

                Logger.log("Set connection options.");

                // Параметры соединения.

                HttpParams params = new BasicHttpParams();
                HttpConnectionParams.setSoTimeout(params, MAX_CLIENT_TIMEOUT);

                ClientConnectionManager cm = new SingleClientConnManager(params, schemeRegistry);
                httpClient = new DefaultHttpClient(cm, params);

                Logger.log("Execute GET request.");

                // GET-запрос.

                HttpGet httpget = new HttpGet(url);
                HttpResponse response = httpClient.execute(httpget);
                HttpEntity entity = response.getEntity();

                Logger.log("Response status: " + response.getStatusLine());
                int status = response.getStatusLine().getStatusCode();

                if (status  != 200) {

                    Logger.log("Bad http response status: " + status);
                    Logger.setStatusFailed();
                    return;

                } // if

                if (entity != null) {

                    Logger.log("Read input stream.");
                    logData(entity.getContent());

                } // if

                Logger.setStatusOK();

            } finally {

                if (httpClient != null) {

                    Logger.log("Shutdown http connection.");
                    Log.i(Constants.APP_LOGGER_TAG, "Shutdown http connection.");

                    // Важно закрыть соединение, т.к. HeapWorker
                    // может завершить работу jvm из-за возможных
                    // задержек в finalize.

                    httpClient.getConnectionManager().shutdown();

                } // if

            }

        }

    }

}
