/**
 * $RCSfileHttpsUrlConnectionExample.java,v $
 * version $Revision: 36379 $
 * created 09.11.2017 11:02 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 * (C) ООО Крипто-Про 2004-2017.
 * <p/>
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 * <p/>
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp.client.example;

import ru.CryptoPro.ACSPClientApp.client.example.base.NoSSLv3SocketFactory;
import ru.CryptoPro.ACSPClientApp.client.example.base.TLSData;
import ru.CryptoPro.ACSPClientApp.client.example.interfaces.ThreadExecuted;
import ru.CryptoPro.ACSPClientApp.util.ContainerAdapter;
import ru.CryptoPro.ACSPClientApp.util.Logger;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;

import java.net.URL;

/**
 * Класс HttpsUrlConnectionExample реализует пример обмена
 * по TLS 1.0 с использованием HttpsUrlConnection.
 *
 * @author Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class HttpsUrlConnectionExample extends TLSData {

    /**
     * Конструктор.
     *
     * @param adapter Настройки примера.
     */
    protected HttpsUrlConnectionExample(ContainerAdapter adapter) {
        super(adapter);
    }

    @Override
    public void getResult() throws Exception {
        getThreadResult(new HttpsUrlConnectionThread());
    }

    /**
     * Класс SimpleTLSThread реализует подключение
     * HttpsUrlConnection клиента по TLS в отдельном
     * потоке.
     *
     */
    private class HttpsUrlConnectionThread implements ThreadExecuted {

        @Override
        public void execute() throws Exception {

            HttpsURLConnection connection = null;
            Logger.log("Init HttpsURLConnection Example.");

            try {

                String httpAddress = containerAdapter.getConnectionInfo().toUrl();
                URL url = new URL(httpAddress);

                SSLContext sslContext = createSSLContext();

                Logger.log("Create socket factory.");
                SSLSocketFactory sslSocketFactory = sslContext.getSocketFactory();

                SSLSocketFactory delegateFactory = new NoSSLv3SocketFactory(sslSocketFactory);
                HttpsURLConnection.setDefaultSSLSocketFactory(delegateFactory);

                Logger.log("Create connection.");

                connection = (HttpsURLConnection) url.openConnection();
                connection.connect();

                Logger.log("Read input stream.");

                logContent(connection);
                Logger.setStatusOK();

            } finally {

                if (connection != null) {
                    connection.disconnect();
                } // if

            }

        }

        /**
         * Вывод полеыенных данных.
         *
         * @param connection Подключение.
         * @throws Exception
         */
        private void logContent(HttpsURLConnection connection)
            throws Exception {

            if (connection != null) {

                Logger.log("Chosen cipher suite: " + connection.getCipherSuite());
                logData(connection.getInputStream());

            } // if

        }

    }

}
