/**
 * Copyright 2004-2013 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp.client.example;

import ru.CryptoPro.ACSPClientApp.client.example.base.TLSData;
import ru.CryptoPro.ACSPClientApp.client.example.interfaces.ThreadExecuted;
import ru.CryptoPro.ACSPClientApp.util.ContainerAdapter;
import ru.CryptoPro.ACSPClientApp.util.Logger;

import java.io.IOException;

/**
 * Класс TLSExample реализует пример обмена
 * по TLS 1.0.
 *
 * 27/05/2013
 *
 */
public class TLSExample extends TLSData {

    /**
     * Конструктор.
     *
     * @param adapter Настройки примера.
     */
    protected TLSExample(ContainerAdapter adapter) {
        super(adapter);
    }

    @Override
    public void getResult() throws Exception {
        getThreadResult(new SampleTLSThread());
    }

    /**
     * Класс SimpleTLSThread реализует подключение
     * самописного клиента по TLS в отдельном потоке.
     *
     */
    private class SampleTLSThread implements ThreadExecuted {

        @Override
        public void execute() throws Exception {

            Logger.log("Init TLS Sample Example.");

            Client client = new Client(
                containerAdapter.getConnectionInfo().getHostAddress(),
                containerAdapter.getConnectionInfo().getHostPort());

            client.setTimeout(MAX_CLIENT_TIMEOUT);

            if (client.get(createSSLContext(),
                containerAdapter.getConnectionInfo().getHostPage()) == 0) {
                Logger.setStatusOK();
            } // if
            else {
                throw new IOException("Couldn't get data.");
            } // else

        }

    }

}
