/**
 * Copyright 2004-2013 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPClientApp.client.example;

import ru.CryptoPro.ACSPClientApp.client.example.base.SignData;
import ru.CryptoPro.ACSPClientApp.client.example.interfaces.ThreadExecuted;
import ru.CryptoPro.ACSPClientApp.util.Constants;
import ru.CryptoPro.ACSPClientApp.util.ContainerAdapter;
import ru.CryptoPro.ACSPClientApp.util.KeyStoreType;
import ru.CryptoPro.ACSPClientApp.util.Logger;

import ru.CryptoPro.JCSP.JCSP;
import java.security.Signature;

/**
 * Класс VerifyExample реализует пример проверки подписи
 * сообщения.
 *
 * 27/05/2013
 *
 */
public class VerifyExample extends SignData {

    /**
     * Конструктор.
     *
     * @param adapter Настройки примера.
     */
    public VerifyExample(ContainerAdapter adapter) {
        super(adapter, false);
    }

    @Override
    public  void getResult() throws Exception {
        getThreadResult(new VerifyThread());
    }

    /**
     * Класс VerifyThread реализует проверку подписи
     * в отдельном потоке.
     *
     */
    private class VerifyThread implements ThreadExecuted {

        @Override
        public void execute() throws Exception {

            Logger.log("Create signature.");

            // Создаем подпись, чтобы потом ее проверить.

            SignExample signData = new SignExample(containerAdapter);
            byte[] sign = signData.sign();

            Logger.log("Load key container to verify signature.");

            // Тип контейнера по умолчанию.

            String keyStoreType = KeyStoreType.currentType();
            Logger.log("Default container type: " + keyStoreType);

            // Загрузка ключа и сертификата.

            load(true, keyStoreType,
                containerAdapter.getClientAlias(),
                containerAdapter.getClientPassword());

            if (getCertificate() == null) {
                Logger.log("Certificate is null.");
                return;
            } // if

            Logger.log("Init Signature: " +
                algorithmSelector.getSignatureAlgorithmName());

            // Инициализация подписи.

            Signature sn = Signature.getInstance(
                algorithmSelector.getSignatureAlgorithmName(),
                JCSP.PROVIDER_NAME);

            Logger.log("Init verification by certificate: " +
                getCertificate().getSubjectDN() + ", public key:" +
                getCertificate().getPublicKey());

            sn.initVerify(getCertificate());

            Logger.log("Source data: " + Constants.MESSAGE);
            sn.update(Constants.MESSAGE.getBytes());

            Logger.log("Verify signature:");
            Logger.log(sign, true);

            // Проверяем подпись.

            if (sn.verify(sign)) {
                Logger.setStatusOK();
            } // if
            else {
                Logger.setStatusFailed();
            } // else

        }

    }

}
