/**
 * Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPInClientApp;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import ru.CryptoPro.ACSPInClientApp.examples.*;
import ru.CryptoPro.ACSPInClientApp.interfaces.OnPageLoadListener;
import ru.CryptoPro.ACSPInClientApp.tools.Logger;

import ru.CryptoPro.JCP.KeyStore.JCPPrivateKeyEntry;

import ru.CryptoPro.JCSP.CSPConfig;
import ru.CryptoPro.JCSP.CSPProviderInterface;
import ru.CryptoPro.JCSP.JCSP;
import ru.CryptoPro.JCSP.support.BKSTrustStore;

import ru.CryptoPro.reprov.RevCheck;
import ru.CryptoPro.ssl.android.util.cpSSLConfig;

import ru.cprocsp.ACSP.tools.common.Constants;

import java.io.File;
import java.security.PrivateKey;
import java.security.Security;
import java.security.cert.X509Certificate;

/**
 * Класс MainActivity главной активити.
 *
 */
public class MainActivity extends Activity {

    @Override
    public void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        // Инициализация лога.

        final TextView tvMainLog = (TextView) findViewById(R.id.tvMainLog);

        Logger.init(tvMainLog);
        Logger.INSTANCE.clear();

        // Инициализация провайдеров.

        if (!initProviders()) {

            Log.e(Constants.APP_LOGGER_TAG, "Couldn't initialize CSP.");
            return;

        } // if

        // Запуск нескольких примеров.

        final Button btRun = (Button) findViewById(R.id.btMainRun);
        btRun.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {

                Logger.INSTANCE.append("Providers initialized. Run examples...");
                btRun.setEnabled(false);

                try {

                    // 1. Проверка целостности.

                    Logger.INSTANCE.append("*** 1. Checking integrity ***");
                    CSPProviderInterface providerInfo = CSPConfig.INSTANCE.getCSPProviderInfo();

                    IntegrityExample integrity = new IntegrityExample(providerInfo);
                    integrity.execute();

                    // 2. Хэш.

                    Logger.INSTANCE.append("*** 2. Hash example ***");

                    HashExample hash = new HashExample();
                    hash.execute();

                    // 3. Ключевой контейнер.

                    Logger.INSTANCE.append("*** 3. Generating example ***");

                    KeyGenerationExample generate = new KeyGenerationExample();
                    JCPPrivateKeyEntry entry = generate.execute();

                    PrivateKey privateKey = entry.getPrivateKey();
                    X509Certificate cert  = (X509Certificate) entry.getCertificate();

                    Logger.INSTANCE.append("Private key: " + privateKey);
                    Logger.INSTANCE.append("Certificate: " + cert.getSubjectDN());

                    // 4. Подпись и проверка.

                    Logger.INSTANCE.append("*** 4. Signing example ***");

                    SignatureExample sign = new SignatureExample(privateKey, cert);
                    sign.execute();

                    // 5. Удаление контейнера.

                    Logger.INSTANCE.append("*** 5. Deleting example ***");

                    KeyDeletingExample delete = new KeyDeletingExample();
                    delete.execute();

                    //
                    // 6. Установка новой и старой лицензии.
                    //
                    // Пример LicenseNewExample не запускаем,
                    // т.к. в нем нет лицензии для проверки.
                    //
                    //
                    // Logger.INSTANCE.append("*** 6. Installing a new license ***");
                    //
                    // LicenseNewExample newLicense = new LicenseNewExample(providerInfo);
                    // newLicense.execute();
                    //

                    Logger.INSTANCE.append("*** 7. Installing standard license ***");

                    LicenseOldExample oldLicense = new LicenseOldExample(providerInfo);
                    oldLicense.execute();

                    // 7. Задание другого типа текущего отделяемого
                    // носителя, отключение/включение типа хранилища,
                    // изменение уровня логирования.

                    Logger.INSTANCE.append("*** 8. Configuring ***");

                    ConfigExample config = new ConfigExample(providerInfo);
                    config.execute();

                    // 8. TLS подключение с использованием фиксированного
                    // хранилища доверенных сертификатов, без аутентификации
                    // клиента.

                    Logger.INSTANCE.append("*** 9. TLS example ***");
                    Thread thread = new Thread(new Runnable() {

                        @Override
                        public void run() {

                            try {

                                HttpsExample tls = new HttpsExample(
                                    getLocalTrustStoreType(),
                                    Environment.getExternalStorageDirectory(),
                                    getLocalTrustStorePath(),
                                    getLocalTrustStorePassword()
                                );

                                // Ожидаем событие загрузки страницы.

                                tls.setOnPageLoadListener(new OnPageLoadListener() {

                                    @Override
                                    public void onPageLoad(File htmlFile) {

                                        Uri uri = Uri.fromFile(htmlFile);

                                        Intent intent = new Intent(Intent.ACTION_VIEW);
                                        intent.setDataAndType(uri, "text/html");

                                        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                                        startActivity(intent);

                                    }

                                });

                                tls.execute();

                            } catch (Exception e) {

                                Log.e(Logger.LOG_TAG, e.getMessage(), e);
                                Logger.INSTANCE.append(e.getMessage());

                            }

                        }

                    });

                    thread.start();
                    thread.join();

                    // 9. Завершение.

                    Logger.INSTANCE.append("*** Completed! ***");

                } catch (Exception e) {

                    Log.e(Logger.LOG_TAG, e.getMessage(), e);
                    Logger.INSTANCE.append(e.getMessage());

                } finally {
                    btRun.setEnabled(true);
                }

            }

        });

    }

    @Override
    public void onResume() {

        super.onResume();

        // Необходимо для отображения диалоговых окон
        // ДСЧ, ввода пин-кода и сообщений.

        CSPConfig.registerActivityContext(this);

    }

    /**
     * Инициализация CSP провайдера. Добавление
     * нативного провайдера Java CSP, SSL-провайдера
     * и Revocation-провайдера в список Security.
     *
     * Происходит один раз при инициализации.
     * Возможно только после инициализации в CSPConfig!
     *
     * @return true в случае успешной инициализации.
     */
    private boolean initProviders() {

        //
        // Инициализация провайдера CSP. Должна выполняться
        // один раз в главном потоке приложения, т.к. использует
        // статические переменные.
        //
        // 1. Создаем инфраструктуру CSP и копируем ресурсы
        // в папку. В случае ошибки мы, например, выводим окошко
        // (или как-то иначе сообщаем) и завершаем работу.
        //

        int initCode  = CSPConfig.init(this);
        if (initCode != CSPConfig.CSP_INIT_OK) {

            Logger.INSTANCE.append("Error occurred during CSP initiation: " + initCode);
            return false;

        } // if

        // Загрузка Java CSP (хеш, подпись, шифрование,
        // генерация контейнеров).

        if (Security.getProvider(JCSP.PROVIDER_NAME) == null) {
            Security.addProvider(new JCSP());
        } // if

        // Загрузка Java TLS (TLS).

        // Необходимо переопределить свойства, чтобы
        // использовались менеджеры из cpSSL, а не Harmony.

        Security.setProperty("ssl.KeyManagerFactory.algorithm",
            ru.CryptoPro.ssl.android.Provider.KEYMANGER_ALG);
        Security.setProperty("ssl.TrustManagerFactory.algorithm",
            ru.CryptoPro.ssl.android.Provider.KEYMANGER_ALG);

        Security.setProperty("ssl.SocketFactory.provider",
            "ru.CryptoPro.ssl.android.SSLSocketFactoryImpl");
        Security.setProperty("ssl.ServerSocketFactory.provider",
            "ru.CryptoPro.ssl.android.SSLServerSocketFactoryImpl");

        if (Security.getProvider(ru.CryptoPro.ssl.android.Provider.PROVIDER_NAME) == null) {
            Security.addProvider(new ru.CryptoPro.ssl.android.Provider());
        } // if

        // Провайдер хеширования, подписи, шифрования
        // по умолчанию.

        cpSSLConfig.setDefaultSSLProvider(JCSP.PROVIDER_NAME);

        // Загрузка Revocation Provider (CRL, OCSP).

        if (Security.getProvider(RevCheck.PROVIDER_NAME) == null) {
            Security.addProvider(new RevCheck());
        } // if

        // Включаем возможность онлайновой проверки
        // статуса сертификата.

        System.setProperty("com.sun.security.enableCRLDP", "true");
        System.setProperty("com.ibm.security.enableCRLDP", "true");

        final String trustStorePath = getLocalTrustStorePath();
        final String trustStorePassword = String.valueOf(getLocalTrustStorePassword());

        Log.d(Constants.APP_LOGGER_TAG, "Default trust store: " + trustStorePath);

        System.setProperty("javax.net.ssl.trustStoreType", getLocalTrustStoreType());
        System.setProperty("javax.net.ssl.trustStore", trustStorePath);
        System.setProperty("javax.net.ssl.trustStorePassword", trustStorePassword);

        return true;

    }

    /**
     * Получение типа хранилища доверенных сертификатов.
     *
     * @return тип хранилища.
     */
    private String getLocalTrustStoreType() {
        return BKSTrustStore.STORAGE_TYPE;
    }

    /**
     * Получение пути к хранилищу доверенных сертификатов.
     *
     * @return путь к хранилищу.
     */
    private String getLocalTrustStorePath() {

        return getApplicationInfo().dataDir + File.separator +
            BKSTrustStore.STORAGE_DIRECTORY + File.separator +
            BKSTrustStore.STORAGE_FILE_TRUST;

    }

    /**
     * Получение пароля к хранилищу доверенных сертификатов.
     *
     * @return пароль к хранилищу.
     */
    private char[] getLocalTrustStorePassword() {
        return BKSTrustStore.STORAGE_PASSWORD;
    }

}