/**
 * $RCSfileConfigExample.java,v $
 * version $Revision: 36379 $
 * created 30.08.2017 14:10 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 * <p/>
 * Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPInClientApp.examples;

import ru.CryptoPro.ACSPInClientApp.interfaces.Executable;
import ru.CryptoPro.ACSPInClientApp.tools.Logger;

import ru.CryptoPro.JCSP.CSPProviderInterface;
import ru.cprocsp.ACSP.tools.config.ConfigInterface;

import java.util.Map;
import java.util.Set;

/**
 * Пример ConfigExample предназначен для
 * управления конфигурацией провайдера.
 *
 * @author Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class ConfigExample implements Executable<Void> {

    /**
     * Параметры провайдера.
     */
    private final CSPProviderInterface providerInfo;

    /**
     * Конструктор.
     *
     * @param providerInfo Параметры провайдера.
     */
    public ConfigExample(CSPProviderInterface providerInfo) {
        this.providerInfo = providerInfo;
    }

    @Override
    public Void execute() throws Exception {

        ConfigInterface config = providerInfo.getConfig();
        Logger.INSTANCE.append("Changing configuration...");

        // Уровень логирования.

        String currentLevel = config.getCurrentLoggingLevel();
        Logger.INSTANCE.append("Current logging level: " + currentLevel);

        // Проверка, отключены ли предупреждения об
        // использовании ГОСТ Р 34.10-2001.

        boolean warning2001Disabled = config.isWarning2001Disabled();
        Logger.INSTANCE.append("Current warning 2001 disabled: " + warning2001Disabled);

        if (warning2001Disabled) {
            throw new Exception("Warning 2001 enabled!");
        } // if

        // Считыватель.

        String currentReaderName = config.getCurrentReaderName();
        Logger.INSTANCE.append("Current reader name: " + currentReaderName);

        // Список считывателей.

        String[] readerList = config.getReaderList();
        Logger.INSTANCE.append("-- Reader list from config --");

        for (String reader : readerList) {
            Logger.INSTANCE.append("reader: " + reader);
        } // for

        // Список типов хранилищ.

        Map<String, String> configKeyStoreTypes = config.getKeyStoreTypes();
        Set<String> keyStoreTypes = configKeyStoreTypes.keySet();

        Logger.INSTANCE.append("-- Key store types from config --");

        for (String keyStoreType : keyStoreTypes) {
            Logger.INSTANCE.append("type: " + keyStoreType);
        } // for

        // Список закешированных типов хранилищ
        // (исходный чистый список). Список можно
        // очистить, а файл кеша удалить. Используется,
        // чтобы не запрашивать список считывателей
        // каждый раз.

        if (config.clearCachedKeyStoreTypes()) {
            Logger.INSTANCE.append("Key store cache cleared.");
        } // if
        else {
            Logger.INSTANCE.append("Couldn't clear key store cache.");
        } // else

        config.getKeyStoreTypes(); // обновление кеша

        Map<String, String> cachedConfigKeyStoreTypes = config.getCachedKeyStoreTypes();
        Set<String> cachedKeyStoreTypes = cachedConfigKeyStoreTypes.keySet();

        Logger.INSTANCE.append("-- Cached key store types --");

        for (String keyStoreType : cachedKeyStoreTypes) {
            Logger.INSTANCE.append("type: " + keyStoreType);
        } // for

        //--------------------------------------------------------------------------------------------------------------

        // Смена уровня логирования.

        String newLoggingLevel = ConfigInterface.LOGGING_LEVEL_NAMES[1];
        Logger.INSTANCE.append("Set logging level " + newLoggingLevel + "...");

        if (config.setLoggingLevel(newLoggingLevel)) {
            Logger.INSTANCE.append(newLoggingLevel + " set.");
        } // if
        else {
            Logger.INSTANCE.append(newLoggingLevel + " NOT set!");
            throw new Exception("Couldn't set logging level");
        } // else

        // Отключение предупреждений об
        // использовании ГОСТ Р 34.10-2001.

        if (config.disableWarning2001(true)) {
            Logger.INSTANCE.append("Warning 2001 disabled.");
        } // if
        else {
            Logger.INSTANCE.append("Warning 2001 still enabled!");
            throw new Exception("Couldn't disable warning 2001");
        } // else

        // Смена считывателя.

        Logger.INSTANCE.append("Looking for reader to change...");
        String newReaderName = null;

        for (String reader : readerList) {
            if (!reader.equals(currentReaderName)) {

                newReaderName = reader;
                break;

            } // if
        } // for

        if (newReaderName != null) {

            Logger.INSTANCE.append("Set reader " + newReaderName + "...");

            if (config.setReaderName(newReaderName)) {
                Logger.INSTANCE.append(newReaderName + " set.");
            } // if
            else {
                Logger.INSTANCE.append(newReaderName + " NOT set!");
                throw new Exception("Couldn't set reader");
            } // else

        } // if

        // Поиск типа для отключения.

        Logger.INSTANCE.append("Looking for key store type to disable...");
        String disabledKeyStoreType = null;

        for (String keyStoreType : keyStoreTypes) {
            if (!config.isKeyStoreAlwaysEnabled(keyStoreType)) {

                disabledKeyStoreType = keyStoreType;
                break;

            }
        } // for

        // Отключение типа.

        if (disabledKeyStoreType != null) {

            Logger.INSTANCE.append("Disable key store type " +
                disabledKeyStoreType + "...");

            if (config.disableKeyStoreType(disabledKeyStoreType)) {
                Logger.INSTANCE.append(disabledKeyStoreType + " disabled.");
            } // if
            else {
                Logger.INSTANCE.append(disabledKeyStoreType + " NOT disabled!");
                throw new Exception("Couldn't disable key store type");
            } // else

        } // if
        else {
            Logger.INSTANCE.append("No key store type available to disable.");
        } // else

        //--------------------------------------------------------------------------------------------------------------

        // Уровень логирования.

        String newCurrentLevel = config.getCurrentLoggingLevel();
        Logger.INSTANCE.append("New logging level: " + newCurrentLevel);

        // Проверка, отключены ли предупреждения.

        boolean newWarning2001Disabled = config.isWarning2001Disabled();
        Logger.INSTANCE.append("New warning2001 disabled: " + newWarning2001Disabled);

        // Считыватель.

        String newCurrentReaderName = config.getCurrentReaderName();
        Logger.INSTANCE.append("New reader name: " + newCurrentReaderName);

        // Список типов хранилищ.

        Map<String, String> newConfigKeyStoreTypes = config.getKeyStoreTypes();
        Set<String> newKeyStoreTypes = newConfigKeyStoreTypes.keySet();

        Logger.INSTANCE.append("-- New key store types from config --");

        for (String keyStoreType : newKeyStoreTypes) {
            Logger.INSTANCE.append("type: " + keyStoreType);
        } // for

        //--------------------------------------------------------------------------------------------------------------

        Logger.INSTANCE.append("Restoring configuration...");

        // Восстановление уровня логирования.

        Logger.INSTANCE.append("Set logging level " + currentLevel + "...");

        if (config.setLoggingLevel(currentLevel)) {
            Logger.INSTANCE.append(currentLevel + " set.");
        } // if
        else {
            Logger.INSTANCE.append(currentLevel + " NOT set!");
            throw new Exception("Couldn't restore logging level");
        } // else

        String restoredLoggingLevel = config.getCurrentLoggingLevel();
        Logger.INSTANCE.append("Restored logging level: " + restoredLoggingLevel);

        // Восстановление предупреждений.

        if (config.disableWarning2001(false)) {
            Logger.INSTANCE.append("Warning 2001 enabled.");
        } // if
        else {
            Logger.INSTANCE.append("Warning 2001 still disabled!");
            throw new Exception("Couldn't restore warning 2001");
        } // else

        boolean restoredWarning2001Disabled = config.isWarning2001Disabled();
        Logger.INSTANCE.append("Restored warning2001 (is disabled): " +
            restoredWarning2001Disabled);

        // Восстановление считывателя.

        if (newReaderName != null) {

            Logger.INSTANCE.append("Set reader " + currentReaderName + "...");

            if (config.setReaderName(currentReaderName)) {
                Logger.INSTANCE.append(currentReaderName + " set.");
            } // if
            else {
                Logger.INSTANCE.append(currentReaderName + " NOT set!");
                throw new Exception("Couldn't restore reader");
            } // else

            String restoredCurrentReaderName = config.getCurrentReaderName();
            Logger.INSTANCE.append("Restored reader name: " + restoredCurrentReaderName);

        } // if

        // Включение типа хранилища.

        if (disabledKeyStoreType != null) {

            Logger.INSTANCE.append("Enable key store type " +
                disabledKeyStoreType + "...");

            if (config.enableKeyStoreType(disabledKeyStoreType)) {
                Logger.INSTANCE.append(disabledKeyStoreType + " enabled.");
            } // if
            else {
                Logger.INSTANCE.append(disabledKeyStoreType + " NOT enabled!");
                throw new Exception("Couldn't enable key store type");
            } // else

        } // if

        //--------------------------------------------------------------------------------------------------------------

        // Уровень логирования.

        newCurrentLevel = config.getCurrentLoggingLevel();
        Logger.INSTANCE.append("New logging level: " + newCurrentLevel);

        // Проверка, отключены ли предупреждения.

        newWarning2001Disabled = config.isWarning2001Disabled();
        Logger.INSTANCE.append("New warning2001 disabled: " + newWarning2001Disabled);

        // Считыватель.

        newCurrentReaderName = config.getCurrentReaderName();
        Logger.INSTANCE.append("New reader name: " + newCurrentReaderName);

        // Список типов хранилищ.

        Map<String, String> restoredConfigKeyStoreTypes = config.getKeyStoreTypes();
        Set<String> restoredKeyStoreTypes = restoredConfigKeyStoreTypes.keySet();

        Logger.INSTANCE.append("-- Restored key store types from config --");

        for (String keyStoreType : restoredKeyStoreTypes) {
            Logger.INSTANCE.append("type: " + keyStoreType);
        } // for

        return null;

    }

}
