/**
 * $RCSfileKeyGenerationExample.java,v $
 * version $Revision: 36379 $
 * created 21.08.2017 13:12 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 * <p/>
 * Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPInClientApp.examples;

import ru.CryptoPro.ACSPInClientApp.interfaces.Executable;
import ru.CryptoPro.ACSPInClientApp.tools.Logger;

import ru.CryptoPro.JCP.KeyStore.JCPPrivateKeyEntry;

import ru.CryptoPro.JCPRequest.GostCertificateRequest;
import ru.CryptoPro.JCSP.JCSP;

import java.io.ByteArrayInputStream;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.KeyStore;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

/**
 * Пример KeyGenerationExample предназначен для
 * создания и сохранения ключа и самоподписанного
 * серитфиката.
 *
 * @author Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class KeyGenerationExample implements Executable<JCPPrivateKeyEntry> {

    /**
     * Алиас ключа.
     */
    public static final String ALIAS = "test_cont";

    /**
     * Пароль к ключу.
     */
    public static final char[] PASSWORD = "123456".toCharArray();

    @Override
    public JCPPrivateKeyEntry execute() throws Exception {

        try {

            KeyDeletingExample delete = new KeyDeletingExample();
            delete.execute();

        } catch (Exception e) {
            // ignore
        }

        Logger.INSTANCE.append("Initializing key pair generator: "
            + KEY_ALGORITHM + "...");

        KeyPairGenerator kpg = KeyPairGenerator.getInstance(
            KEY_ALGORITHM, JCSP.PROVIDER_NAME);

        Logger.INSTANCE.append("Generating key pair...");
        KeyPair keyPair = kpg.generateKeyPair();

        Logger.INSTANCE.append("Key pair completed. Generating self-signed certificate...");
        X509Certificate selfSignedCertStub = getSelfSignedCertStub(keyPair, SIG_ALGORITHM);

        Logger.INSTANCE.append("Saving key pair...");

        KeyStore keyStore = KeyStore.getInstance(
            JCSP.HD_STORE_NAME, JCSP.PROVIDER_NAME);

        keyStore.load(null, null);

        KeyStore.ProtectionParameter protectedParam =
            new KeyStore.PasswordProtection(PASSWORD);

        JCPPrivateKeyEntry entry = new JCPPrivateKeyEntry(
            keyPair.getPrivate(),
            new Certificate[] {selfSignedCertStub}
        );

        keyStore.setEntry(ALIAS, entry, protectedParam);

        Logger.INSTANCE.append("Saving completed, password: " +
            String.copyValueOf(PASSWORD) + ". Initializing key store...");

        keyStore = KeyStore.getInstance(JCSP.HD_STORE_NAME, JCSP.PROVIDER_NAME);
        keyStore.load(null, null);

        Logger.INSTANCE.append("Reading private key and certificate...");
        entry = (JCPPrivateKeyEntry) keyStore.getEntry(ALIAS, protectedParam);

        Logger.INSTANCE.append("Reading completed.");
        return entry;

    }

    /**
     * Создание самоподписанного сертификата.
     *
     * @param keyPair Ключевая пара.
     * @param algorithm Алгоритм подписи.
     * @return сертификат.
     * @throws Exception
     */
    private X509Certificate getSelfSignedCertStub(KeyPair
        keyPair, String algorithm) throws Exception {

        GostCertificateRequest request = new GostCertificateRequest(JCSP.PROVIDER_NAME);
        String dn = "CN=" + ALIAS;

        Logger.INSTANCE.append("Creating self-signed certificate: "
            + algorithm + "...");

        byte[] certContent = request.getEncodedSelfCert(keyPair,
            dn, algorithm);

        Logger.INSTANCE.append("Creating completed. Converting" +
            " certificate to X.509...");

        X509Certificate cert = (X509Certificate) CertificateFactory.getInstance("X.509")
            .generateCertificate(new ByteArrayInputStream(certContent));

        Logger.INSTANCE.append("Converting completed.");
        return cert;

    }

}
