/**
 * $RCSfileLicenseNewExample.java,v $
 * version $Revision: 36379 $
 * created 28.08.2017 11:11 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 * <p/>
 * Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package ru.CryptoPro.ACSPInClientApp.examples;

import ru.CryptoPro.ACSPInClientApp.interfaces.Executable;
import ru.CryptoPro.ACSPInClientApp.tools.Logger;

import ru.CryptoPro.JCSP.CSPProviderInterface;
import ru.cprocsp.ACSP.tools.common.CSPLicenseConstants;
import ru.cprocsp.ACSP.tools.license.LicenseInterface;

/**
 * Пример LicenseNewExample предназначен для
 * установки новой лицензии.
 *
 * @author Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class LicenseNewExample implements Executable<Void> {

    /**
     * Новая лицензия CSP.
     */
    private static final String LICENSE_NEW = ""; // Лицензия CSP

    /**
     * Параметры провайдера.
     */
    private final CSPProviderInterface providerInfo;

    /**
     * Конструктор.
     *
     * @param providerInfo Параметры провайдера.
     */
    public LicenseNewExample(CSPProviderInterface providerInfo) {
        this.providerInfo = providerInfo;
    }

    /**
     * Получение номера лицензии.
     *
     * @return номер лицензии.
     */
    public String getNumber() {
        return LICENSE_NEW;
    }

    @Override
    public Void execute() throws Exception {

        // Проверка текущей лицензии.

        LicenseInterface license = providerInfo.getLicense();
        String number = license.getSerialNumber();

        Logger.INSTANCE.append("Checking current license "
            + number + "...");

        license.checkAndSave();
        Logger.INSTANCE.append("Checking completed.");

        printLicenseInfo(license);

        // Проверка новой лицензии.

        Logger.INSTANCE.append("Installing license: " + getNumber() + "...");

        license.checkAndSave(getNumber(), false);
        Logger.INSTANCE.append("Installing completed.");

        printLicenseInfo(license);
        return null;

    }

    /**
     * Вывод информации о лицензии.
     *
     * @param license Лицензия.
     * @throws Exception
     */
    private void printLicenseInfo(LicenseInterface
        license) throws Exception {

        int result = license.getLicenseStatus();

        Logger.INSTANCE.append("Result: " + result);
        int licenseType = license.getLicenseType();

        if (result == CSPLicenseConstants.LICENSE_STATUS_OK) {
            Logger.INSTANCE.append("Status: OK");
        } // if
        else {

            if (licenseType == CSPLicenseConstants.LICENSE_TYPE_EXPIRED) {
                Logger.INSTANCE.append("Status: expired");
            } // if
            else {
                Logger.INSTANCE.append("Status: invalid");
            } // else

        } // else

        if (result != CSPLicenseConstants.LICENSE_STATUS_INVALID) {

            if (licenseType == CSPLicenseConstants.LICENSE_TYPE_EXPIRED) {
                Logger.INSTANCE.append("expiredThroughDays: expired");
            } // if
            else if (licenseType == CSPLicenseConstants.LICENSE_TYPE_PERMANENT) {
                Logger.INSTANCE.append("expiredThroughDays: permanent");
            } // else if
            else {
                Logger.INSTANCE.append("expiredThroughDays: " +
                    license.getExpiredThroughDays());
            } // else

        } // if

        Logger.INSTANCE.append("Installation date: " +
            license.getLicenseInstallDateAsString());

        if (result != CSPLicenseConstants.LICENSE_STATUS_OK) {
            throw new Exception("Could not check or/and install license.");
        } // if

    }

}
