/**
 * $RCSfileTLSProxy.java,v $
 * version $Revision: 36379 $
 * created 15.08.2016 17:13 by afevma
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 * <p/>
 * Copyright 2004-2016 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package tls_proxy;

import java.util.Arrays;

/**
 * Класс TLSProxy предназначен для запуска
 * прокси.
 *
 * @author Copyright 2004-2016 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class TLSProxy {

    /**
     * Пример конфигурации.
     */
    private static final String PROXY_CONFIG_EXAMPLE =
    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
    + "\n<Config>"
    + "\n<!-- Parameter 'provider' is key store provider. -->"
    + "\n<Parameters inactiveTimeout=\"60\" checkInactiveTimeout=\"30\" serverSoTimeout=\"600\" provider=\"JCP\" protocol=\"GostTLS\"/>"
    + "\n<!-- 'path' to trust store with 'type' CertStore and 'password'. It can"
    + " be absolute path to the store file or file name. If file name set "
    + "path to the config.xml is used. 'provider' is store provider name.-->"
    + "\n<CertStore path=\"c:\\software\\Keys\\tomcat7\\test_ca.store\" password=\"1\" type=\"CertStore\" provider=\"JCP\" />"
    + "\n<!-- List of hosts -->"
    + "\n<Addresses>"
    + "\n<Address>"
    + "\n<!-- Listening local port. -->"
    + "\n<ListenPort>9000</ListenPort>"
    + "\n<!-- Host parameters: host and port. -->"
    + "\n<Host>cpca.cryptopro.ru</Host>"
    + "\n<Port>443</Port>"
    + "\n<!-- True if client authentication required. "
    + "If true then KeyType and KeyPassword must be set. Also keyAlias is available. -->"
    + "\n<ClientAuthEnabled>false</ClientAuthEnabled>"
    + "\n</Address>"
    + "\n<Address>"
    + "\n<ListenPort>9001</ListenPort>"
    + "\n<Host>cryptopro.ru</Host>"
    + "\n<Port>4444</Port>"
    + "\n<ClientAuthEnabled>true</ClientAuthEnabled>"
    + "\n<KeyType>HDImageStore</KeyType>"
    + "\n<KeyPassword>Pass1234</KeyPassword>"
    + "\n</Address>"
    + "\n</Addresses>"
    + "\n</Config>";

    public static void main(String[] args) throws Exception {

        System.setProperty("com.sun.security.enableCRLDP", "true");
        System.setProperty("com.ibm.security.enableCRLDP", "true");

        if (args.length == 0) {
            System.out.println(
                  "Usage:"
                + "\ntls_proxy -help to get an example of configuration"
                + "\ntls_proxy <listen-port> <path to config.xml> to start application"
                + "\ntls_proxy <listen-port> to start application"
                + "\nExamples:"
                + "\njava -jar tls_proxy.jar 9000"
                + "\njava -jar tls_proxy.jar 9001 C:/config.xml"
            );
            return;
        } // if

        if (Arrays.asList(args).contains("-help")) {
            System.out.println("Example of configuration file 'config.xml':\n");
            System.out.println(PROXY_CONFIG_EXAMPLE);
            return;
        } // if

        int listenPort = Integer.parseInt(args[0]);
        MainLogger.info("*** Listen port: " + listenPort);

        String configFile = null;
        if (args.length > 1) {
            configFile = args[1];
        } // if

        ConfigReader.init(configFile);
        ConnectionManager connectionManager = new ConnectionManager(listenPort);

        connectionManager.start();
        connectionManager.join();

    }

}
