/**
 * Copyright 2004-2012 Crypto-Pro. All rights reserved.
 * Программный код, содержащийся в этом файле, предназначен
 * для целей обучения. Может быть скопирован или модифицирован
 * при условии сохранения абзацев с указанием авторства и прав.
 *
 * Данный код не может быть непосредственно использован
 * для защиты информации. Компания Крипто-Про не несет никакой
 * ответственности за функционирование этого кода.
 */
package userSamples.ca15;

import ru.CryptoPro.JCPRequest.ca15.decoder.CA15CertificateRequestRecord;
import ru.CryptoPro.JCPRequest.ca15.request.CA15GostCertificateRequest;
import ru.CryptoPro.JCPRequest.ca15.user.CA15User;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

/**
 * Пример получения списка запросов на сертификаты зарегистрированного
 * пользователя по протоколу HTTPS (CA15).
 *
 * 10/11/2012
 *
 */
public class GetUserCertificateRequestListExample extends CAConfiguration {


    /**
     * Конструктор.
     *
     * @param needInit Флаг необходимости выполнить
     * инициализацию.
     */
    protected GetUserCertificateRequestListExample(boolean needInit) {
        super(needInit);
    }

    /**
     *
     * @param args
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {

        final GetUserCertificateRequestListExample example = new GetUserCertificateRequestListExample(true);
        example.main();

    }

    @Override
    public void main() throws Exception {

        // Зарегистрированный пользователь, чьи запросы на сертификат мы
        // хотим получить. Если пользователь не зарегистрирован, то его
        // следует предварительно зарегистрировать с помощью примера
        // RegisterUserExample.
        CA15User userInfo = new CA15User(TOKEN_ID, PASSWORD);

        // Получаем список запросов на сертификаты.
        Map<String, CA15CertificateRequestRecord> requestMap =
            CA15GostCertificateRequest.getCertificateRequestList(
                CA15GostCertificateRequest.TEST_CA15_HTTPS_ADDRESS, userInfo);

        Collection<CA15CertificateRequestRecord> requests = requestMap.values();
        Iterator requestIterator = requests.iterator();

        // Выводим список запросов.
        while (requestIterator.hasNext()) {

            CA15CertificateRequestRecord requestRecord =
                (CA15CertificateRequestRecord) requestIterator.next();

            System.out.print("Request N: " + requestRecord.getCertificateRequestIdentifier());
            System.out.print("\n\tstatus: " + requestRecord.getStatus().toString());
            System.out.print("\n\tsent date: " + requestRecord.getSentDate());
            System.out.print("\n\tapproval date: " + requestRecord.getApprovalDate());
            System.out.println("\n\tcomment: " + requestRecord.getComment());
        }

        // Список может быть пустым, например, для только что зарегистрированного
        // пользователя, а также пользователя с истекшим маркером или несуществующего
        // пользователя.
        if (requestMap.size() == 0) {
            System.out.println("User " + userInfo.getTokenID() + " has empty table.");
        }
    }

}
