/**
 * $RCSfileAnonymousResolver.java,v $
 * version $Revision: 36379 $
 * created 01.02.2018 16:22 by elvira
 * last modified $Date: 2012-05-30 12:19:27 +0400 (Ср, 30 май 2012) $ by $Author: afevma $
 * <p/>
 * Copyright 2004-2018 Crypto-Pro. All rights reserved. Этот файл содержит
 * информацию, являющуюся собственностью компании Крипто-Про.
 * <p/>
 * Любая часть этого файла не может быть скопирована, исправлена, переведена на
 * другие языки, локализована или модифицирована любым способом,
 * откомпилирована, передана по сети с или на любую компьютерную систему без
 * предварительного заключения соглашения с компанией Крипто-Про.
 */
package xades;

import org.apache.xml.security.signature.XMLSignatureInput;
import org.apache.xml.security.utils.resolver.ResourceResolverException;
import org.apache.xml.security.utils.resolver.ResourceResolverSpi;

import org.w3c.dom.Attr;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

/**
 * Класс AnonymousResolver реализует ResourceResolver и используется для
 * элементов, у которых отсутствует URI.
 *
 * @author Copyright 2004-2017 Crypto-Pro. All rights reserved.
 * @.Version
 */
public class AnonymousResolver extends ResourceResolverSpi {

    private InputStream inputStream = null;
    private byte[] bytes = null;
    private String fileName = null;

    /**
     * Конструктор на основе байтового массива.
     *
     * @param data Данные.
     */
    public AnonymousResolver(byte[] data) {
        this.bytes = data;
    }

    /**
     * Конструктор на основе потока данных.
     *
     * @param stream Данные.
     */
    public AnonymousResolver(InputStream stream) {
        this.inputStream = stream;
    }

    /**
     * Конструктор на основе пути к файлу.
     * Используется только в однопоточном режиме.
     * В многопоточном режиме небезопасен!
     *
     * @param fileName Путь к файлу с данными.
     */
    public AnonymousResolver(String fileName) {
        this.fileName = fileName;
    }

    @Override
    public XMLSignatureInput engineResolve(Attr uri, String BaseURI)
        throws ResourceResolverException {

        XMLSignatureInput result = null;

        if (bytes != null) {
            result = new XMLSignatureInput(bytes);
        } // if
        else if(fileName != null) {

            try {
                result = new XMLSignatureInput(new FileInputStream(fileName));
            } catch (FileNotFoundException e) {
                throw new ResourceResolverException(e.getMessage(), uri, BaseURI);
            }

        } // else
        else if (inputStream != null) {
            result = new XMLSignatureInput(inputStream);
        } // else

        if (result != null) {
            result.setExcludeComments(true);
            result.setMIMEType("text/xml");
        }

        return result;

    }

    @Override
    public boolean engineCanResolve(Attr uri, String BaseURI) {

        if (uri == null) {
            return true;
        }

        return false;

    }

    @Override
    public boolean engineIsThreadSafe() {
        return true;
    }

}
